@@ -115,6 +115,8 @@ struct BusConfigParser
 
   DBusList *service_dirs; /**< Directories to look for services in */
 
+  DBusList *conf_dirs;   /**< Directories to look for policy configuration in */
+
   BusPolicy *policy;     /**< Security policy */
 
   BusLimits limits;      /**< Limits */
@@ -322,6 +324,9 @@ merge_included (BusConfigParser *parser,
 
   while ((link = _dbus_list_pop_first_link (&included->service_dirs)))
     _dbus_list_append_link (&parser->service_dirs, link);
+
+  while ((link = _dbus_list_pop_first_link (&included->conf_dirs)))
+    _dbus_list_append_link (&parser->conf_dirs, link);
   
   return TRUE;
 }
@@ -419,7 +424,7 @@ bus_config_parser_new (const DBusString      *basedir,
       parser->limits.max_pending_activations = 256;
       parser->limits.max_services_per_connection = 256;
       
-      parser->limits.max_match_rules_per_connection = 128;
+      parser->limits.max_match_rules_per_connection = 512;
       
       parser->limits.reply_timeout = 5 * 60 * 1000; /* 5 minutes */
       parser->limits.max_replies_per_connection = 32;
@@ -468,6 +473,12 @@ bus_config_parser_unref (BusConfigParser *parser)
 
       _dbus_list_clear (&parser->service_dirs);
 
+      _dbus_list_foreach (&parser->conf_dirs,
+                          (DBusForeachFunction) dbus_free,
+                          NULL);
+
+      _dbus_list_clear (&parser->conf_dirs);
+
       _dbus_list_foreach (&parser->mechanisms,
                           (DBusForeachFunction) dbus_free,
                           NULL);
@@ -1677,7 +1688,7 @@ set_limit (BusConfigParser *parser,
       must_be_positive = TRUE;
       parser->limits.max_message_size = value;
     }
-  else if (strcmp (name, "activation_timeout") == 0)
+  else if (strcmp (name, "service_start_timeout") == 0)
     {
       must_be_positive = TRUE;
       must_be_int = TRUE;
@@ -1713,13 +1724,13 @@ set_limit (BusConfigParser *parser,
       must_be_int = TRUE;
       parser->limits.max_connections_per_user = value;
     }
-  else if (strcmp (name, "max_pending_activations") == 0)
+  else if (strcmp (name, "max_pending_service_starts") == 0)
     {
       must_be_positive = TRUE;
       must_be_int = TRUE;
       parser->limits.max_pending_activations = value;
     }
-  else if (strcmp (name, "max_services_per_connection") == 0)
+  else if (strcmp (name, "max_names_per_connection") == 0)
     {
       must_be_positive = TRUE;
       must_be_int = TRUE;
@@ -1965,6 +1976,7 @@ include_dir (BusConfigParser   *parser,
   dbus_bool_t retval;
   DBusError tmp_error;
   DBusDirIter *dir;
+  char *s;
   
   if (!_dbus_string_init (&filename))
     {
@@ -2021,7 +2033,21 @@ include_dir (BusConfigParser   *parser,
       dbus_move_error (&tmp_error, error);
       goto failed;
     }
-  
+
+
+  if (!_dbus_string_copy_data (dirname, &s))
+    {
+      BUS_SET_OOM (error);
+      goto failed;
+    }
+
+  if (!_dbus_list_append (&parser->conf_dirs, s))
+    {
+      dbus_free (s);
+      BUS_SET_OOM (error);
+      goto failed;
+    }
+
   retval = TRUE;
   
  failed:
@@ -2358,6 +2384,12 @@ bus_config_parser_get_service_dirs (BusConfigParser *parser)
   return &parser->service_dirs;
 }
 
+DBusList**
+bus_config_parser_get_conf_dirs (BusConfigParser *parser)
+{
+  return &parser->conf_dirs;
+}
+
 dbus_bool_t
 bus_config_parser_get_fork (BusConfigParser   *parser)
 {
@@ -2814,28 +2846,29 @@ all_are_equiv (const DBusString *target_directory)
       printf ("    %s\n", _dbus_string_get_const_data (&filename));
 
       parser = bus_config_load (&full_path, TRUE, NULL, &error);
-      _dbus_string_free (&full_path);
 
       if (parser == NULL)
 	{
 	  _dbus_warn ("Could not load file %s: %s\n",
 		      _dbus_string_get_const_data (&full_path),
 		      error.message);
+          _dbus_string_free (&full_path);
 	  dbus_error_free (&error);
 	  goto finished;
 	}
       else if (first_parser == NULL)
 	{
+          _dbus_string_free (&full_path);
 	  first_parser = parser;
 	}
       else
 	{
+          _dbus_string_free (&full_path);
 	  equal = config_parsers_equal (first_parser, parser);
 	  bus_config_parser_unref (parser);
 	  if (! equal)
 	    goto finished;
 	}
-
     }
 
   retval = TRUE;
