@@ -33,6 +33,9 @@ DBUS_BEGIN_DECLS
 
 DBusConnection *dbus_bus_get              (DBusBusType     type,
 					   DBusError      *error);
+DBusConnection *dbus_bus_get_private      (DBusBusType     type,
+					   DBusError      *error);
+
 dbus_bool_t     dbus_bus_register         (DBusConnection *connection,
 					   DBusError      *error);
 dbus_bool_t     dbus_bus_set_unique_name  (DBusConnection *connection,
@@ -45,6 +48,9 @@ int             dbus_bus_request_name     (DBusConnection *connection,
 					   const char     *name,
 					   unsigned int    flags,
 					   DBusError      *error);
+int             dbus_bus_release_name     (DBusConnection *connection,
+					   const char     *name,
+					   DBusError      *error);
 dbus_bool_t     dbus_bus_name_has_owner   (DBusConnection *connection,
 					   const char     *name,
 					   DBusError      *error);
