@@ -32,6 +32,18 @@
 extern "C" {
 #endif
 
+/* Normally docs are in .c files, but there isn't a .c file for this. */
+/**
+ * @defgroup DBusShared Shared constants 
+ * @ingroup  DBus
+ *
+ * Shared constants.
+ *
+ * @brief Stuff used by both dbus/dbus.h low-level and C/C++ binding APIs 
+ * @{
+ */
+
+
 typedef enum
 {
   DBUS_BUS_SESSION,    /**< The login session bus */
@@ -61,34 +73,36 @@ typedef enum
 #define DBUS_INTERFACE_PROPERTIES     "org.freedesktop.DBus.Properties"
 #define DBUS_INTERFACE_PEER           "org.freedesktop.DBus.Peer"
 
-/* This is a special interface whose methods can only be invoked
+/** This is a special interface whose methods can only be invoked
  * by the local implementation (messages from remote apps aren't
  * allowed to specify this interface).
  */
 #define DBUS_INTERFACE_LOCAL "org.freedesktop.DBus.Local"
 
 /* Owner flags */
-#define DBUS_NAME_FLAG_ALLOW_REPLACEMENT 0x1
-#define DBUS_NAME_FLAG_REPLACE_EXISTING  0x2
-#define DBUS_NAME_FLAG_DO_NOT_QUEUE      0x4
+#define DBUS_NAME_FLAG_ALLOW_REPLACEMENT 0x1 /**< Allow another service to become the primary owner if requested */
+#define DBUS_NAME_FLAG_REPLACE_EXISTING  0x2 /**< Request to replace the current primary owner */
+#define DBUS_NAME_FLAG_DO_NOT_QUEUE      0x4 /**< If we can not become the primary owner do not place us in the queue */
 
 /* Replies to request for a name */
-#define DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER  1
-#define DBUS_REQUEST_NAME_REPLY_IN_QUEUE       2
-#define DBUS_REQUEST_NAME_REPLY_EXISTS         3
-#define DBUS_REQUEST_NAME_REPLY_ALREADY_OWNER  4
+#define DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER  1 /**< Service has become the primary owner of the requested name */
+#define DBUS_REQUEST_NAME_REPLY_IN_QUEUE       2 /**< Service could not become the primary owner and has been placed in the queue */
+#define DBUS_REQUEST_NAME_REPLY_EXISTS         3 /**< Service is already in the queue */
+#define DBUS_REQUEST_NAME_REPLY_ALREADY_OWNER  4 /**< Service is already the primary owner */
 
 /* Replies to releasing a name */
-#define DBUS_RELEASE_NAME_REPLY_RELEASED        1
-#define DBUS_RELEASE_NAME_REPLY_NON_EXISTENT    2
-#define DBUS_RELEASE_NAME_REPLY_NOT_OWNER       3
+#define DBUS_RELEASE_NAME_REPLY_RELEASED        1 /**< Service was released from the given name */
+#define DBUS_RELEASE_NAME_REPLY_NON_EXISTENT    2 /**< The given name does not exist on the bus */
+#define DBUS_RELEASE_NAME_REPLY_NOT_OWNER       3 /**< Service is not an owner of the given name */
 
 /* Replies to service starts */
-#define DBUS_START_REPLY_SUCCESS         1
-#define DBUS_START_REPLY_ALREADY_RUNNING 2
+#define DBUS_START_REPLY_SUCCESS         1 /**< service was auto started */
+#define DBUS_START_REPLY_ALREADY_RUNNING 2 /**< service was already running */
+
+/** @} */
 
 #ifdef __cplusplus
 }
 #endif
 
-#endif /* DBUS_PROTOCOL_H */
+#endif /* DBUS_SHARED_H */
