@@ -1,6 +1,8 @@
 /* qdbusmarshall.h QDBusMarshall object
  *
  * Copyright (C) 2005 Harald Fernengel <harry@kdevelop.org>
+ * Copyright (C) 2006 Trolltech AS. All rights reserved.
+ *    Author: Thiago Macieira <thiago.macieira@trolltech.com>
  *
  * Licensed under the Academic Free License version 2.1
  *
@@ -15,8 +17,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, write to the Free Software Foundation
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
 
@@ -25,14 +27,15 @@
 
 struct DBusMessage;
 
+template <typename T> class QList;
 class QVariant;
-template <typename T>
-class QList;
+class QString;
 
 class QDBusMarshall
 {
 public:
-    static void listToMessage(const QList<QVariant> &list, DBusMessage *message);
+    static void listToMessage(const QList<QVariant> &list, DBusMessage *message,
+                              const QString& signature);
     static void messageToList(QList<QVariant> &list, DBusMessage *message);
 };
 
