@@ -1,6 +1,8 @@
 /* qdbusmessage.h QDBusMessage private object
  *
  * Copyright (C) 2005 Harald Fernengel <harry@kdevelop.org>
+ * Copyright (C) 2006 Trolltech AS. All rights reserved.
+ *    Author: Thiago Macieira <thiago.macieira@trolltech.com>
  *
  * Licensed under the Academic Free License version 2.1
  *
@@ -15,17 +17,16 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, write to the Free Software Foundation
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
 
 #ifndef QDBUSMESSAGE_P_H
 #define QDBUSMESSAGE_P_H
 
-#include <QtCore/qatomic.h>
-#include <QtCore/qstring.h>
-
+#include <qatomic.h>
+#include <qstring.h>
 struct DBusMessage;
 
 class QDBusMessagePrivate
@@ -34,13 +35,15 @@ public:
     QDBusMessagePrivate(QDBusMessage *qq);
     ~QDBusMessagePrivate();
 
-    QString path, interface, name, service, method, sender;
+    QString service, path, interface, name, message, signature;
     DBusMessage *msg;
     DBusMessage *reply;
     QDBusMessage *q;
     int type;
     int timeout;
     QAtomic ref;
+
+    mutable bool repliedTo : 1;
 };
 
 #endif
